"use strict";

var request = require("request");
var Promise = require("bluebird");
var _ = require("lodash");

var ROOT = "https://bitbucket.org/api/";

var getP = Promise.promisify(request.get);
var postP = Promise.promisify(request.post);

function get (path, options) {
  return getP(_.merge({url: ROOT + path}, options || {})).spread(function (res, body) {
    if (_.isString(body) && body.length > 0) {
      return JSON.parse(body);
    }
  });
}

function post (path, data, options) {
  return postP(_.merge({url: ROOT + path, json: data}, options || {})).spread(function (res, body) {
    return body;
  });
}

function log (prefix) {
  return function (v) {
    console.log(prefix, v);
    return v;
  };
}

function createOAuth (username, password) {
  return get("1.0/users/" + username + "/consumers", {
      "auth": {
        "user": username,
        "pass": password
      }
    })
    .then(log("CONSUMERS"))
    .then(_.partialRight(_.findLast, {"name": "github-todos"}))
    .then(log("FOUND CONSUMER"))
    .then(function (found) {
      if (found) {
        return found;
      }

      return post("1.0/users/" + username + "/consumers", {
        "name":         "github-todos",
        "description":  "Github-Todos CLI"
      }, {
        "auth": {
          "user": username,
          "pass": password
        }
      })
      .then(log("NEW CONSUMER"));
    });
}

createOAuth("naholyr", "bb_abghrt;2504")
.then(function (oauth) {
  return get("1.0/repositories/naholyr/sparkleshare-host/issues", {
    "oauth": {
      "consumer_key":     oauth.key,
      "consumer_secret":  oauth.secret
    }
  })
  .then(log("ISSUES"));
})
